from __future__ import unicode_literals
from .base_node_renderer import BaseNodeRenderer


class NullRenderer(BaseNodeRenderer):
    def render(self, node):
        raise Exception(
            "No renderer defined for '{0}' nodes".format(self._node_type(node))
        )

    def _node_type(self, node):
        if "nodeType" in node:
            return node["nodeType"]
        elif "type" in node:
            return node["type"]
        return node



from __future__ import unicode_literals
from copy import deepcopy

from .base_node_renderer import BaseNodeRenderer


class TextRenderer(BaseNodeRenderer):
    def render(self, node):
        node = deepcopy(node)
        for mark in node.get("marks", []):
            renderer = self._find_renderer(mark)

            if renderer is not None:
                node["value"] = renderer.render(node)
        return node["value"]

    def _find_renderer(self, node):
        renderer = self.mappings.get(node.get("type", None), None)

        if renderer is not None:
            return renderer(self.mappings)


class BaseInlineRenderer(BaseNodeRenderer):
    def render(self, node):
        return "<{0}>{1}</{0}>".format(self._render_tag, node["value"])

    @property
    def _render_tag(self):
        return "span"


class BoldRenderer(BaseInlineRenderer):
    @property
    def _render_tag(self):
        return "b"


class CodeRenderer(BaseInlineRenderer):
    @property
    def _render_tag(self):
        return "code"


class ItalicRenderer(BaseInlineRenderer):
    @property
    def _render_tag(self):
        return "i"


class UnderlineRenderer(BaseInlineRenderer):
    @property
    def _render_tag(self):
        return "u"



from __future__ import unicode_literals

from .text_renderers import (
    TextRenderer,
    BoldRenderer,
    CodeRenderer,
    ItalicRenderer,
    UnderlineRenderer,
)
from .block_renderers import (
    HrRenderer,
    HeadingOneRenderer,
    HeadingTwoRenderer,
    HeadingThreeRenderer,
    HeadingFourRenderer,
    HeadingFiveRenderer,
    HeadingSixRenderer,
    BlockQuoteRenderer,
    HyperlinkRenderer,
    ListItemRenderer,
    OrderedListRenderer,
    UnorderedListRenderer,
    ParagraphRenderer,
    EntryBlockRenderer,
    AssetBlockRenderer,
    AssetHyperlinkRenderer,
    TableCellRenderer,
    TableHeaderCellRenderer,
    TableRowRenderer,
    TableRenderer
)
from .document_renderers import DocumentRenderer
from .null_renderer import NullRenderer
from .base_node_renderer import BaseNodeRenderer


class RichTextRenderer(BaseNodeRenderer):
    def __init__(self, mappings=None):
        DEFAULT_MAPPINGS = {
            "document": DocumentRenderer,
            "heading-1": HeadingOneRenderer,
            "heading-2": HeadingTwoRenderer,
            "heading-3": HeadingThreeRenderer,
            "heading-4": HeadingFourRenderer,
            "heading-5": HeadingFiveRenderer,
            "heading-6": HeadingSixRenderer,
            "blockquote": BlockQuoteRenderer,
            "hyperlink": HyperlinkRenderer,
            "list-item": ListItemRenderer,
            "ordered-list": OrderedListRenderer,
            "unordered-list": UnorderedListRenderer,
            "hr": HrRenderer,
            "embedded-entry-block": EntryBlockRenderer,
            "embedded-asset-block": AssetBlockRenderer,
            "asset-hyperlink": AssetHyperlinkRenderer,
            "paragraph": ParagraphRenderer,
            "text": TextRenderer,
            "bold": BoldRenderer,
            "code": CodeRenderer,
            "italic": ItalicRenderer,
            "underline": UnderlineRenderer,
            "table-cell": TableCellRenderer,
            "table-row": TableRowRenderer,
            "table-header-cell": TableHeaderCellRenderer,
            "table": TableRenderer,
            None: NullRenderer,
        }

        self.mappings = DEFAULT_MAPPINGS
        if mappings is not None:
            self.mappings.update(mappings)

    def render(self, document):
        return self._find_renderer(document).render(document)



from __future__ import unicode_literals
from .base_node_renderer import BaseNodeRenderer


class DocumentRenderer(BaseNodeRenderer):
    def render(self, document):
        result = []
        for node in document["content"]:
            renderer = self._find_renderer(node)

            if renderer is None:
                continue

            result.append(renderer.render(node))

        return "\n".join(result)
